﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32;
using Winfoot_7_SDK;

namespace WinfootDefaultSystemPlugin.Controls.Strategy
{
    public interface IMinAnimateStrategy
    {
        void Write(int i);
        WFRegistry Read();
        List<WFRegistry> BackUp();
    }

    public class MinAnimateDefaultStrategy : ICorrespondenceOS, IMinAnimateStrategy
    {
        public MinAnimateDefaultStrategy()
        {
            CorrespondenceOSList = new List<OS>
                                       {
                                           OS.WindowsXP,
                                           OS.WindoesXP_SP1,
                                           OS.WindowsXP_SP2,
                                           OS.WindowsXP_SP3,

#if DEBUG
                                           OS.WindowsVista,
                                           OS.WindowsVista_SP1,
                                           OS.WindowsSeven
#endif
                                       };
        }

        private string RegName = "EncryptionContextMenu";

        #region ICorrespondenceOS メンバ

        public List<OS> CorrespondenceOSList { get; set; }

        #endregion

        #region IAdditionalWorkerThreadsStrategy メンバ

        public void Write(int i)
        {
            RegistryKey registryKey =
                Registry.CurrentUser.CreateSubKey(
                    @"Control Panel\Desktop\WindowMetrics");

            registryKey.SetValue(RegName, i);
        }

        public WFRegistry Read()
        {
            RegistryKey registryKey =
                Registry.CurrentUser.OpenSubKey(
                    @"Control Panel\Desktop\WindowMetrics");

            return new WFRegistry(@"HKEY_CURRENT_USER\Control Panel\Desktop\WindowMetrics", RegName,
                              registryKey.GetValue(RegName, 0), RegistryValueKind.DWord, RegOperation.KeyWrite);



        }

        public List<WFRegistry> BackUp()
        {
            return new List<WFRegistry> { Read() };
        }

        #endregion
    }

    public class MinAnimateController
    {
        public MinAnimateController(MinAnimate minAnimate,
                                                         IMinAnimateStrategy strategy)
        {
            MinAnimate = minAnimate;
            Strategy = strategy;
        }

        public MinAnimate MinAnimate { get; set; }
        public IMinAnimateStrategy Strategy { get; set; }

        public void Read()
        {
            MinAnimate.SetEncryptionContextMenuState(Convert.ToBoolean(Strategy.Read().Value));
        }

        public void Write()
        {
            Strategy.Write(MinAnimate.GetEncryptionContextMenuState());
        }

        public List<WFRegistry> BackUp()
        {
            return Strategy.BackUp();
        }
    }

    class MinAnimateStrategy
    {
    }
}
